@@ -1,12 +1,18 @@
 import json
+import logging
 import os
 import pprint
 import sys
-import warnings
-from ichrome.exceptions import ChromeRuntimeError
-import typer
+from pathlib import Path
+from typing import Annotated, Any, Callable, List, Optional
 
-from ssc_codegen.ast_builder import build_ast_module
+import typer
+from ichrome.exceptions import ChromeRuntimeError
+from typer import Argument, BadParameter, Option, Typer
+
+from ssc_codegen.ast_build import build_ast_module_parser
+from ssc_codegen.cli.ast_grep import JS_RULES, PY_RULES
+from ssc_codegen.cli.cli_callbacks import cb_check_ssc_files, cb_folder_out
 from ssc_codegen.cli.cli_utils import (
     ConverterLike,
     create_fmt_cmd,
@@ -16,21 +22,15 @@ from ssc_codegen.cli.cli_utils import (
 )
 from ssc_codegen.cli.code_callbacks import (
     CB_PY_CODE,
-    CB_DART_CODE,
     CB_GO_CODE,
     CB_JS_CODE,
 )
-from ssc_codegen.cli.runtime_parse_runners import (
-    assert_cls_target,
-    parse_from_html_file,
-    print_json_output,
-    parse_from_http_request,
-    parse_from_chrome,
-)
-from .consts import (
+from ssc_codegen.cli.consts import (
+    CMD_LUA,
+    HELP_LUA_PCRE_FLAG,
+    LuaLIBS,
     PyLIBS,
     JsLIBS,
-    DartLIBS,
     GoLIBS,
     COMMENT_STRING,
     HELP_OUTPUT_FOLDER,
@@ -44,21 +44,28 @@ from .consts import (
     CMD_VERSION,
     CMD_PY,
     CMD_JS,
-    CMD_DART,
     CMD_GO,
     CMD_JSON_GEN,
     DEFAULT_UA,
+    HELP_DOCSTRING,
 )
-from ..converters.json_to_schema import convert_json_to_schema_code
-from .cli_callbacks import cb_check_ssc_files, cb_folder_out
+from ssc_codegen.cli.runtime_parse_runners import (
+    assert_cls_target,
+    parse_from_html_file,
+    print_json_output,
+    parse_from_http_request,
+    parse_from_chrome,
+)
+from ssc_codegen.json_to_scc import convert_json_to_schema_code
 
+app = Typer(
+    no_args_is_help=True,
+    pretty_exceptions_enable=False,
+    pretty_exceptions_short=False,
+    pretty_exceptions_show_locals=False,
+)
 
-from pathlib import Path
-from typing import Annotated, Callable, List, Optional
-
-from typer import Argument, BadParameter, Option, Typer
-
-app = Typer(no_args_is_help=True)
+LOGGER = logging.getLogger("ssc_gen")
 
 
 @app.command(help=CMD_VERSION)
@@ -78,50 +85,59 @@ def generate_code(
     comment_str: str,
     fmt_cmd: list[str],
     code_cb: Callable[[list[str]], str] = lambda c: "\n".join(c),
-    docstring_class_top: bool = False,
-    variables_patches: dict[str, str] | None = None,
+    variables_patches: dict[str, Any] | None = None,
     css_to_xpath: bool = False,
     xpath_to_css: bool = False,
     debug_instructions: bool = False,
-    debug_comment_prefix: str = "",
+    gen_docstring: bool = True,
 ) -> None:
     """universal generate code entrypoint"""
     variables_patches = variables_patches or {}
     if css_to_xpath and xpath_to_css:
         raise BadParameter("Should be passed to-xpath or to-css flag")
     if css_to_xpath:
-        print("Convert ALL CSS queries to XPATH")
+        LOGGER.info("Convert ALL CSS queries to XPATH")
     elif xpath_to_css:
-        print("Convert ALL XPATH queries to CSS")
+        LOGGER.info("Convert ALL XPATH queries to CSS")
 
-    print("Generating code start")
+    for k, v in variables_patches.items():
+        setattr(converter, k, v)
+
+    LOGGER.info("Generating code start")
     if debug_instructions:
-        print("TOGGLE debug generated tokens")
-        converter.set_debug_prefix(debug_comment_prefix)
+        LOGGER.info("TOGGLE debug generated tokens")
+        converter.debug_instructions = True
     for file_cfg in ssc_files:
+        # check ssc-gen files
+        if (
+            file_cfg.suffix != ".py"
+            and "ssc_codegen" not in file_cfg.read_text()
+        ):
+            LOGGER.info("Skip %s (not ssc-gen config)", file_cfg.name)
+            continue
         name = file_cfg.name.split(".")[0]
         out_file = f"{prefix}{name}{suffix}"
-        print(f"Make AST {file_cfg.name}...")
-        ast_module = build_ast_module(
+        LOGGER.info("Make AST %s...", file_cfg.name)
+        ast_module = build_ast_module_parser(
             file_cfg,
-            docstring_class_top=docstring_class_top,
             css_to_xpath=css_to_xpath,
             xpath_to_css=xpath_to_css,
+            gen_docstring=gen_docstring,
         )
-        print(f"Convert to code {file_cfg.name}...")
+        LOGGER.info("Convert to code %s...", file_cfg.name)
         code_parts = converter.convert_program(ast_module, comment=comment_str)
         code = code_cb(code_parts)
-        for k, v in variables_patches.items():
-            code = code.replace(f"${k}$", v)
+        # for k, v in variables_patches.items():
+        #     code = code.replace(f"${k}$", v)
         out_path = out / out_file
-        print(f"save {str(out_path)}")
+        LOGGER.info("save generated code to %s", str(out_path))
         with open(out_path, "w") as f:
             f.write(code)
     if fmt_cmd:
-        print("format code")
+        LOGGER.info("run formatters code")
         for cmd in fmt_cmd:
             os.system(cmd)
-    print("done")
+    LOGGER.info("done")
 
 
 @app.command("py", help=CMD_PY)
@@ -149,13 +165,21 @@ def gen_py(
     debug: Annotated[
         bool, Option(help=HELP_DEBUG_COMM_TOKENS, is_flag=True)
     ] = False,
+    docstring: Annotated[
+        bool, Option(help=HELP_DOCSTRING, is_flag=True)
+    ] = True,
 ) -> None:
     converter = import_converter(f"py_{lib.value}")
     if fmt:
-        commands = ["ruff format {}", "ruff check {} --fix"]
+        commands = [
+            "ruff format {}",
+            "ruff check {} --fix",
+            f"ast-grep scan --rule {PY_RULES} -U " + "{}",
+        ]
         fmt_cmd = create_fmt_cmd(ssc_files, prefix, suffix, out, commands)
     else:
         fmt_cmd = []
+
     generate_code(
         converter=converter,
         out=out,
@@ -167,8 +191,8 @@ def gen_py(
         xpath_to_css=to_css,
         css_to_xpath=to_xpath,
         debug_instructions=debug,
-        debug_comment_prefix="# ",
         code_cb=CB_PY_CODE,
+        gen_docstring=docstring,
     )
 
 
@@ -197,11 +221,16 @@ def gen_js(
     debug: Annotated[
         bool, Option(help=HELP_DEBUG_COMM_TOKENS, is_flag=True)
     ] = False,
+    docstring: Annotated[
+        bool, Option(help=HELP_DOCSTRING, is_flag=True)
+    ] = True,
 ) -> None:
     converter = import_converter(f"js_{lib.value}")
     if fmt:
-        # TODO: js formatters
-        commands: list[str] = []
+        commands: list[str] = [
+            f"ast-grep scan --rule {JS_RULES} -U " + "{}",
+            "prettier --write --tab-width 4 --single-quote --trailing-comma all --semi {}",
+        ]
         fmt_cmd = create_fmt_cmd(ssc_files, prefix, suffix, out, commands)
     else:
         fmt_cmd = []
@@ -213,61 +242,11 @@ def gen_js(
         suffix=suffix,
         comment_str=f"// {COMMENT_STRING}",
         fmt_cmd=fmt_cmd,
-        docstring_class_top=True,
         xpath_to_css=to_css,
         css_to_xpath=to_xpath,
         debug_instructions=debug,
-        debug_comment_prefix="// ",
         code_cb=CB_JS_CODE,
-    )
-
-
-@app.command("dart", help=CMD_DART)
-def gen_dart(
-    ssc_files: Annotated[
-        List[Path],
-        Argument(help="ssc-gen config files", callback=cb_check_ssc_files),
-    ],
-    out: Annotated[
-        Path,
-        Option("--out", "-o", help=HELP_OUTPUT_FOLDER, callback=cb_folder_out),
-    ],
-    lib: Annotated[
-        DartLIBS, Option("--lib", "-i", help=HELP_CORE_LIB)
-    ] = DartLIBS.UNIVERSAL_HTML,
-    prefix: Annotated[
-        str, Option("--prefix", "-p", help=HELP_FILE_PREFIX)
-    ] = "",
-    suffix: Annotated[
-        str, Option("--suffix", "-s", help=HELP_FILE_SUFFIX)
-    ] = ".dart",
-    fmt: Annotated[bool, Option(help=HELP_FMT, is_flag=True)] = True,
-    to_xpath: Annotated[bool, Option(help=HELP_TO_XPATH, is_flag=True)] = False,
-    to_css: Annotated[bool, Option(help=HELP_TO_CSS, is_flag=True)] = False,
-    debug: Annotated[
-        bool, Option(help=HELP_DEBUG_COMM_TOKENS, is_flag=True)
-    ] = False,
-) -> None:
-    converter = import_converter(f"dart_{lib.value}")
-    if fmt:
-        commands = ["dart format {}"]
-        fmt_cmd = create_fmt_cmd(ssc_files, prefix, suffix, out, commands)
-    else:
-        fmt_cmd = []
-    generate_code(
-        converter=converter,
-        out=out,
-        prefix=prefix,
-        ssc_files=ssc_files,
-        suffix=suffix,
-        comment_str=f"// {COMMENT_STRING}",
-        fmt_cmd=fmt_cmd,
-        code_cb=CB_DART_CODE,
-        docstring_class_top=True,
-        xpath_to_css=to_css,
-        css_to_xpath=to_xpath,
-        debug_instructions=debug,
-        debug_comment_prefix="// ",
+        gen_docstring=docstring,
     )
 
 
@@ -298,6 +277,9 @@ def gen_go(
     to_xpath: Annotated[bool, Option(help=HELP_TO_XPATH, is_flag=True)] = False,
     to_css: Annotated[bool, Option(help=HELP_TO_CSS, is_flag=True)] = False,
     debug: Annotated[bool, Option(help=HELP_DEBUG_COMM_TOKENS)] = False,
+    docstring: Annotated[
+        bool, Option(help=HELP_DOCSTRING, is_flag=True)
+    ] = True,
 ) -> None:
     converter = import_converter(f"go_{lib.value}")
     if fmt:
@@ -315,12 +297,64 @@ def gen_go(
         fmt_cmd=fmt_cmd,
         # todo: better API for code callbacks
         code_cb=CB_GO_CODE,
-        docstring_class_top=True,
         variables_patches={"PACKAGE": package or out.name},
         xpath_to_css=to_css,
         css_to_xpath=to_xpath,
         debug_instructions=debug,
-        debug_comment_prefix="// ",
+        gen_docstring=docstring,
+    )
+
+
+@app.command("lua", help=CMD_LUA)
+def gen_lua(
+    ssc_files: Annotated[
+        List[Path],
+        Argument(help="ssc-gen config files", callback=cb_check_ssc_files),
+    ],
+    out: Annotated[
+        Path,
+        Option("--out", "-o", help=HELP_OUTPUT_FOLDER, callback=cb_folder_out),
+    ],
+    lib: Annotated[
+        LuaLIBS, Option("--lib", "-i", help=HELP_CORE_LIB)
+    ] = LuaLIBS.LUA_HTMLPARSER,
+    prefix: Annotated[
+        str, Option("--prefix", "-p", help=HELP_FILE_PREFIX)
+    ] = "",
+    suffix: Annotated[
+        str, Option("--suffix", "-s", help=HELP_FILE_SUFFIX)
+    ] = ".lua",
+    use_pcre: Annotated[
+        bool, Option(help=HELP_LUA_PCRE_FLAG, is_flag=True)
+    ] = False,
+    fmt: Annotated[bool, Option(help=HELP_FMT, is_flag=True)] = True,
+    to_xpath: Annotated[bool, Option(help=HELP_TO_XPATH, is_flag=True)] = False,
+    to_css: Annotated[bool, Option(help=HELP_TO_CSS, is_flag=True)] = False,
+    debug: Annotated[bool, Option(help=HELP_DEBUG_COMM_TOKENS)] = False,
+    docstring: Annotated[
+        bool, Option(help=HELP_DOCSTRING, is_flag=True)
+    ] = True,
+) -> None:
+    converter = import_converter(f"lua_{lib.value}")
+    if fmt:
+        # TODO UNTESTED
+        commands = ["lua-format {}"]
+        fmt_cmd = create_fmt_cmd(ssc_files, prefix, suffix, out, commands)
+    else:
+        fmt_cmd = []
+    generate_code(
+        converter=converter,
+        out=out,
+        prefix=prefix,
+        ssc_files=ssc_files,
+        suffix=suffix,
+        comment_str=f"-- {COMMENT_STRING}",
+        fmt_cmd=fmt_cmd,
+        variables_patches={"USE_PCRE": use_pcre},
+        xpath_to_css=to_css,
+        css_to_xpath=to_xpath,
+        debug_instructions=debug,
+        gen_docstring=docstring,
     )
 
 
@@ -344,6 +378,10 @@ def json_to_schema(
         str,
         Option("--name", "-n", help="first struct name entrypoint"),
     ] = "Content",
+    json_start_path: Annotated[
+        str,
+        Option("--json-path", "-jp", help="optional json_start_path target"),
+    ] = "",
 ) -> None:
     if isinstance(jsn_file, Path):
         jsn_text = jsn_file.read_text(encoding="utf-8").strip()
@@ -356,7 +394,9 @@ def json_to_schema(
         msg = f"Expected dict but got {type(jsn).__name__}"
         raise typer.BadParameter(msg)
 
-    code = convert_json_to_schema_code(jsn, entrypoint_name)
+    code = convert_json_to_schema_code(
+        jsn, entrypoint_name, json_start_path=json_start_path
+    )
     out.write_text(
         "# json-struct: Autogenerated by ssc-gen\n"
         + "from ssc_codegen import Json\n"
@@ -382,11 +422,11 @@ def parse_from_file(
     ] = None,
 ) -> None:
     from ssc_codegen.compiler import Compiler
-    from ssc_codegen.converters.py_parsel import converter
+    from ssc_codegen.converters.py_parsel import CONVERTER
 
     cls_target, schema_config = _validate_parser_target(cls_target)
 
-    compiler = Compiler.from_file(schema_config, converter=converter)
+    compiler = Compiler.from_file(schema_config, converter=CONVERTER)
     if not assert_cls_target(compiler, cls_target):
         suggest = suggest_class_name(compiler._module, cls_target)  # noqa
         msg = f"{schema_config.name} not contains {cls_target} class. Did you mean `{suggest}`?"
@@ -401,7 +441,8 @@ def parse_from_file(
     if out:
         if isinstance(result, str):
             result = json.loads(result)
-        json.dump(result, out, indent=2)
+        with open(out, "w") as f:
+            json.dump(result, f, indent=2)
     else:
         print_json_output(result)
 
@@ -437,11 +478,11 @@ def parse_from_url(
     ] = True,
 ) -> None:
     from ssc_codegen.compiler import Compiler
-    from ssc_codegen.converters.py_parsel import converter
+    from ssc_codegen.converters.py_bs4 import CONVERTER
 
     cls_target, schema_config = _validate_parser_target(cls_target)
 
-    compiler = Compiler.from_file(schema_config, converter=converter)
+    compiler = Compiler.from_file(schema_config, converter=CONVERTER)
     if not assert_cls_target(compiler, cls_target):
         suggest = suggest_class_name(compiler._module, cls_target)  # noqa
         msg = f"{schema_config.name} not contains {cls_target} class. Did you mean `{suggest}`?"
@@ -458,7 +499,8 @@ def parse_from_url(
     if out:
         if isinstance(result, str):
             result = json.loads(result)
-        json.dump(result, out, indent=2)
+        with open(out, "w") as f:
+            json.dump(result, f, indent=2)
     else:
         print_json_output(result)
 
@@ -501,14 +543,14 @@ def parse_from_chrome_(
         ),
     ] = "",
 ) -> None:
-    from ssc_codegen.converters.js_pure import converter as js_converter
+    from ssc_codegen.converters.js_pure import CONVERTER as JS_CONVERTER
     import asyncio
     from ssc_codegen.compiler import Compiler
-    from ssc_codegen.converters.py_parsel import converter
+    from ssc_codegen.converters.py_parsel import CONVERTER
 
     cls_target, schema_config = _validate_parser_target(cls_target)
 
-    compiler = Compiler.from_file(schema_config, converter=converter)
+    compiler = Compiler.from_file(schema_config, converter=CONVERTER)
     if not assert_cls_target(compiler, cls_target):
         suggest = suggest_class_name(compiler._module, cls_target)  # noqa
         msg = f"{schema_config.name} not contains {cls_target} class. Did you mean `{suggest}`?"
@@ -520,8 +562,8 @@ def parse_from_chrome_(
     if not url.startswith("http"):
         raise typer.BadParameter(f"`{url}` not a http url")
 
-    ast = build_ast_module(schema_config, docstring_class_top=True)  # type: ignore
-    code_parts = js_converter.convert_program(ast)
+    ast = build_ast_module_parser(schema_config)  # type: ignore
+    code_parts = JS_CONVERTER.convert_program(ast)
     code = CB_JS_CODE(code_parts)
     code += f"; JSON.stringify((new {cls_target}(document).parse()))"
 
@@ -541,7 +583,9 @@ def parse_from_chrome_(
         )
     except ChromeRuntimeError as e:
         msg = f"{e} try manually provide chrome executable path"
-        warnings.warn(msg, category=Warning)
+        logging.error(
+            "Not founded chromium executable. Try manually provide chrome executable path by `-sc` option"
+        )
         exit(1)
     try:
         if isinstance(result, str):
@@ -552,7 +596,7 @@ def parse_from_chrome_(
         else:
             print(result)
     except Exception as e:
-        print("parse json error", e)
+        logging.error("parse json error", exc_info=e)
         pprint.pprint(result, sort_dicts=False)
 
 
